
(function ($) {
	
	/**
	 * Add new 'script' transport for 'script' datatypes and 'jsonp' where
	 * it is both cross-domain and the url length is greater than the url
	 * length limit.
	 * 
	 */
   errorFields = "";
	jQuery.ajaxTransport("+script", function( s, o /* originalSettings */, jqXHR) {
		var MAXIMUM_URL_LENGTH = 2048;
		
		var isMsie = true;
		if (jQuery.browser) {
			isMsie = jQuery.browser.msie != undefined;
		} else {
			var ua = navigator.userAgent.toLowerCase();
			var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
				/(webkit)[ \/]([\w.]+)/.exec(ua) ||
				/(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
				/(msie) ([\w.]+)/.exec(ua) ||
				ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
				[];
			isMsie = match[1] == 'msie';
		}
		
		if (isMsie) {
			MAXIMUM_URL_LENGTH = 2048; // for ie
		} else {
			MAXIMUM_URL_LENGTH = 8175; // for Apache
		}

		// This transport only deals with cross domain requests
		if (s.canChunk && s.crossDomain && s.url.length > MAXIMUM_URL_LENGTH) {
			var script,
				head = document.head || document.getElementsByTagName("head")[0] || document.documentElement;

			return {

				send: function(_, callback) {

					var ajaxLocParts, callbackName, encodedQueryString, ts,
						rhash = /#.*$/,
						rjsonp = /(=)\?(?=&|$)|\?\?/,
						rprotocol = /^\/\//,
						rts = /(^|&)_=[^&]*/,
						//rts = /([?&])_=[^&]*/,
						rurl = /^([\w\+\.\-]+:)(?:\/\/([^\/?#:]*)(?::(\d+)|)|)/,
						hasCallback = s.jsonp !== false,
						queryString = '',
						data = '',
						url = o.url;

					ajaxLocParts = rurl.exec(jQuery.ajaxSettings.url.toLowerCase()) || [];
					url = (url + "").replace(rhash, "").replace(rprotocol, ajaxLocParts[1] + "//");

					if (url.indexOf('?') !== -1) {
						queryString = url.slice(url.indexOf('?')+1); // must be first
						url = url.slice(0, url.indexOf('?'));
					}

					// Convert data if not already a string
					if ( o.data && s.processData && typeof o.data !== "string" ) {
						data = jQuery.param(o.data, o.traditional);
					} else {
						data = o.data;
					}

					if (queryString === '') {
						queryString = data;
					} else {
						queryString += '&'+data;
					}

					// Insert callback into url or form data
					if (hasCallback && rjsonp.test(queryString)) {
						queryString = queryString.replace(rjsonp, "$1"+s.jsonpCallback);
					} else if (hasCallback) {
						if (queryString === '') {
							queryString = s.jsonp+"="+s.jsonpCallback;
						} else {
							queryString += '&'+s.jsonp+"="+s.jsonpCallback;
						}
					}

					if (s.cache === false) {
						// Remove anti-cache if present because we will add it later
						queryString = queryString.replace(rts, ''); 
						// Remove the & character if first character of the queryString
						queryString = queryString.replace(/^&/, ''); 
					}

					// encode again
					encodedQueryString = encodeURIComponent(queryString);

					var chunkedResponse, calculatedLength, overwritten, script, 
						chunkCount = 0,
						sendChunkedFirstTime = true,
						fixedLength = 
							url.length + 
							('?_c=1').length + // 1) chunked flag '_c=1'
							('&_='+jQuery.now()).length + // 2) jquery.now timestamp '&_=1368725033656'
							('&'+s.jsonp+"="+s.jsonpCallback+'_c').length + // 3) callback '&callback=jQuery18209600335203576833_1368652483694_c'
							('&_cd=').length,
						usableCharactersLength = MAXIMUM_URL_LENGTH - fixedLength,
						totalChunkCount = Math.ceil(encodedQueryString.length / usableCharactersLength);

					function onReadyStateChange (_, isAbort) {
						if (sendChunkedFirstTime) {
							// Get the overwritten function and install callback
							overwritten = window[s.jsonpCallback+'_c'];
							window[s.jsonpCallback+'_c'] = function() {
								chunkedResponse = arguments;
							};
						}

						if (sendChunkedFirstTime || isAbort || !script.readyState || /loaded|complete/.test(script.readyState)) {
							var encodedQueryStringChunk, fullUrl;

							if (!sendChunkedFirstTime) {
								if (s.chunkStatus)
									s.chunkStatus((chunkCount)/totalChunkCount, {args: chunkedResponse});

								// handle response here
								chunkedResponse;

								// Handle memory leak in IE
								script.onload = script.onreadystatechange = null;

								// Remove the script
								if ( head && script.parentNode ) {
									head.removeChild( script );
								}

								// Dereference the script
								script = undefined;
							} else {
								if (s.chunkStatus)
									s.chunkStatus(0, {args: 'Start'});
								sendChunkedFirstTime = false;
							}

							if (window.debuggerStop) {
								return;
							}

							calculatedLength = 
								fixedLength + // everything that doesn't change
								encodedQueryString.length; // 5) chunked data

							if (calculatedLength > MAXIMUM_URL_LENGTH) { // need to chunk
								// get a chunk of encodedQueryString
								encodedQueryStringChunk = encodedQueryString.slice(0, usableCharactersLength);

								// status is _c=1
								fullUrl = 
									url + 
									'?_c=1' + // tell server we are still going
									'&_='+jQuery.now() + 
									'&'+s.jsonp+"="+s.jsonpCallback+'_c' + // use a madeup callback
									'&_cd='+encodedQueryStringChunk; // use the encodedQueryStringChunk

								// shorten encodedQueryString
								encodedQueryString = encodedQueryString.slice(usableCharactersLength);
								
								chunkCount++;

								// transfer
								script = document.createElement("script");
								script.async = "async";
								if ( s.scriptCharset ) {
									script.charset = s.scriptCharset;
								}
								script.src = fullUrl;
								script.onload = script.onreadystatechange = onReadyStateChange;

								// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
								// This arises when a base node is used (#2709 and #4378).
								head.insertBefore( script, head.firstChild );
							} else if (encodedQueryString.length > 0) { // send the last/only chunk
								// restore the function we used
								window[s.jsonpCallback+'_c'] = overwritten;

								// status is _c=0
								fullUrl =
									url + 
									'?_c=0' + // tell server we are done
									'&_='+jQuery.now() + 
									'&'+s.jsonp+"="+s.jsonpCallback + //use the real callback now
									'&_cd='+encodedQueryString; // use the whole encodedQueryString since it is the right size
								
								chunkCount++;

								// set encodedQueryString to ''
								encodedQueryString = '';

								// transfer
								script = document.createElement("script");
								script.async = "async";
								if ( s.scriptCharset ) {
									script.charset = s.scriptCharset;
								}
								script.src = fullUrl;
								script.onload = script.onreadystatechange = onReadyStateChange;

								// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
								// This arises when a base node is used (#2709 and #4378).
								head.insertBefore( script, head.firstChild );
							} else { //clean up
								// restore the function we used
								window[s.jsonpCallback+'_c'] = overwritten;

								// Callback if not abort
								if ( !isAbort ) {
									callback( 200, "success" );
								}
							}

						}
					}

					// kick off the process
					onReadyStateChange();

					// done
				},

				abort: function() {
					if ( script ) {
						script.onload( 0, 1 );
					}
				}
			};

		}
	});

	/**
	 * jQuery yiiactiveform plugin file.
	 *
	 * @author Qiang Xue <qiang.xue@gmail.com>
	 * @link http://www.yiiframework.com/
	 * @copyright Copyright &copy; 2008-2010 Yii Software LLC
 	 * @license http://www.yiiframework.com/license/
	 * @version $Id: jquery.yiiactiveform.js 3538 2012-01-14 18:24:30Z mdomba $
	 * @since 1.1.1
	 */

	/*
	 * returns the value of the CActiveForm input field
	 * performs additional checks to get proper values for checkbox / radiobutton / checkBoxList / radioButtonList
	 * @param o object the jQuery object of the input element
	 */
	var getAFValue = function (o) {
		var type,
			c = [];
		if (!o.length) {
			return undefined;
		}
		if (o[0].tagName.toLowerCase() === 'span') {
			o.find(':checked').each(function () {
				c.push(this.value);
			});
			return c.join(',');
		}
		type = o.attr('type');
		if (type === 'checkbox' || type === 'radio') {
			return o.filter(':checked').val();
		} else {
			return o.val();
		}
	};

	/**
	 * yiiactiveform set function.
	 * @param options map settings for the active form plugin. Please see {@link CActiveForm::options} for availablel options.
	 */
	$.fn.yiiactiveform = function (options) {
		return this.each(function () {
			var settings = $.extend({}, $.fn.yiiactiveform.defaults, options || {}),
				$form = $(this);

			if (settings.validationUrl === undefined) {
				settings.validationUrl = $form.attr('action');
			}
			$.each(settings.attributes, function (i) {
				this.value = getAFValue($form.find('#' + this.inputID));
				settings.attributes[i] = $.extend({}, {
					validationDelay: settings.validationDelay,
					validateOnChange: settings.validateOnChange,
					validateOnType: settings.validateOnType,
					hideErrorMessage: settings.hideErrorMessage,
					inputContainer: settings.inputContainer,
					errorCssClass: settings.errorCssClass,
					successCssClass: settings.successCssClass,
					beforeValidateAttribute: settings.beforeValidateAttribute,
					afterValidateAttribute: settings.afterValidateAttribute,
					validatingCssClass: settings.validatingCssClass
				}, this);
			});
			$form.data('settings', settings);

			settings.submitting = false;  // whether it is waiting for ajax submission result
			var validate = function (attribute, forceValidate) {
				if (forceValidate) {
					attribute.status = 2;
				}
				$.each(settings.attributes, function () {
					if (this.value !== getAFValue($form.find('#' + this.inputID))) {
						this.status = 2;
						forceValidate = true;
					}
				});
				if (!forceValidate) {
					return;
				}

				if (settings.timer !== undefined) {
					clearTimeout(settings.timer);
				}
				settings.timer = setTimeout(function () {
					if (settings.submitting || $form.is(':hidden')) {
						return;
					}
					if (attribute.beforeValidateAttribute === undefined || attribute.beforeValidateAttribute($form, attribute)) {
						$.each(settings.attributes, function () {
							if (this.status === 2) {
								this.status = 3;
								$.fn.yiiactiveform.getInputContainer(this, $form).addClass(this.validatingCssClass);
							}
						});
						$.fn.yiiactiveform.validate($form, function (data) {
							var hasError = false;
							$.each(settings.attributes, function () {
								if (this.status === 2 || this.status === 3) {
									hasError = $.fn.yiiactiveform.updateInput(this, data, $form) || hasError;
								}
							});
							if (attribute.afterValidateAttribute !== undefined) {
								attribute.afterValidateAttribute($form, attribute, data, hasError);
							}
						});
					}
				}, attribute.validationDelay);
			};

			$.each(settings.attributes, function (i, attribute) {
				if (this.validateOnChange) {
					$form.find('#' + this.inputID).change(function () {
						validate(attribute, false);
					}).blur(function () {
						if (attribute.status !== 2 && attribute.status !== 3) {
							validate(attribute, !attribute.status);
						}
					});
				}
				if (this.validateOnType) {
					$form.find('#' + this.inputID).keyup(function () {
						if (attribute.value !== getAFValue($(this))) {
							validate(attribute, false);
						}
					});
				}
			});

			if (settings.validateOnSubmit) {
				$form.find(':submit').live('mouseup keyup', function () {
					$form.data('submitObject', $(this));
				});
				var validated = false;
				$form.submit(function () {
					if (validated) {
						return true;
					}
					if (settings.timer !== undefined) {
						clearTimeout(settings.timer);
					}
					settings.submitting = true;
					if (settings.beforeValidate === undefined || settings.beforeValidate($form)) {
						$.fn.yiiactiveform.validate($form, function (data) {
							var hasError = false;
							$.each(settings.attributes, function () {
								hasError = $.fn.yiiactiveform.updateInput(this, data, $form) || hasError;
							});
							$.fn.yiiactiveform.updateSummary($form, data);
							if (settings.afterValidate === undefined || settings.afterValidate($form, data, hasError)) {
								if (!hasError) {
									validated = true;
									var $button = $form.data('submitObject') || $form.find(':submit:first');
									// TODO: if the submission is caused by "change" event, it will not work
									if ($button.length) {
										$button.click();
									} else {  // no submit button in the form
										$form.submit();
									}
									return;
								}
							}
							settings.submitting = false;
						});
					} else {
						settings.submitting = false;
					}
					return false;
				});
			}

			/*
			 * In case of reseting the form we need to reset error messages
			 * NOTE1: $form.reset - does not exist
			 * NOTE2: $form.live('reset', ...) does not work
			 */
			$form.bind('reset', function () {
				/*
				 * because we bind directly to a form reset event, not to a reset button (that could or could not exist),
				 * when this function is executed form elements values have not been reset yet,
				 * because of that we use the setTimeout
				 */
				setTimeout(function () {
					$.each(settings.attributes, function () {
						this.status = 0;
						var $error = $form.find('#' + this.errorID),
							$container = $.fn.yiiactiveform.getInputContainer(this, $form);

						$container.removeClass(
							this.validatingCssClass + ' ' +
							this.errorCssClass + ' ' +
							this.successCssClass
						);

						$error.html('').hide();

						/*
						 * without the setTimeout() we would get here the current entered value before the reset instead of the reseted value
						 */
						this.value = getAFValue($form.find('#' + this.inputID));
					});
					/*
					 * If the form is submited (non ajax) with errors, labels and input gets the class 'error'
					 */
					$form.find('label, input').each(function () {
						$(this).removeClass('error');
					});
					$('#' + settings.summaryID).hide().find('ul').html('');
					//.. set to initial focus on reset
					if (settings.focus !== undefined && !window.location.hash) {
						$form.find(settings.focus).focus();
					}
				}, 1);
			});

			/*
			 * set to initial focus
			 */
			if (settings.focus !== undefined && !window.location.hash) {
				$form.find(settings.focus).focus();
			}
		});
	};

	/**
	 * Returns the container element of the specified attribute.
	 * @param attribute object the configuration for a particular attribute.
	 * @param form the form jQuery object
	 * @return jquery the jquery representation of the container
	 */
	$.fn.yiiactiveform.getInputContainer = function (attribute, form) {
		if (attribute.inputContainer === undefined) {
			return form.find('#' + attribute.inputID).closest('div');
		} else {
			return form.find(attribute.inputContainer).filter(':has("#' + attribute.inputID + '")');
		}
	};

	/**
	 * updates the error message and the input container for a particular attribute.
	 * @param attribute object the configuration for a particular attribute.
	 * @param messages array the json data obtained from the ajax validation request
	 * @param form the form jQuery object
	 * @return boolean whether there is a validation error for the specified attribute
	 */
	$.fn.yiiactiveform.updateInput = function (attribute, messages, form) {
		attribute.status = 1;
		var $error, $container,
			hasError = false,
			$el = form.find('#' + attribute.inputID);
		if ($el.length) {
			hasError = messages !== null && $.isArray(messages[attribute.id]) && messages[attribute.id].length > 0;
			$error = form.find('#' + attribute.errorID);
			$container = $.fn.yiiactiveform.getInputContainer(attribute, form);

			$container.removeClass(
				attribute.validatingCssClass + ' ' + 
				attribute.errorCssClass + ' ' + 
				attribute.successCssClass
			);

			if (hasError) {
				$error.html(messages[attribute.id][0]);
				$container.addClass(attribute.errorCssClass);
        if(errorFields == ""){
            errorFields = attribute.name;
        } else{
            errorFields = errorFields +"|"+attribute.name;
        }
			} else if (attribute.enableAjaxValidation || attribute.clientValidation) {
				$container.addClass(attribute.successCssClass);
			}
			if (!attribute.hideErrorMessage) {
				$error.toggle(hasError);
			}

			attribute.value = getAFValue($el);
		}
		return hasError;
	};

	/**
	 * updates the error summary, if any.
	 * @param form jquery the jquery representation of the form
	 * @param messages array the json data obtained from the ajax validation request
	 */
	$.fn.yiiactiveform.updateSummary = function (form, messages) {
		var settings = $(form).data('settings'),
			content = '';
		if (settings.summaryID === undefined) {
			return;
		}
		if (messages) {
			$.each(settings.attributes, function () {
				if ($.isArray(messages[this.id])) {
					$.each(messages[this.id], function (j, message) {
						content = content + '<li>' + message + '</li>';
					});
				}
			});
		}
		$('#' + settings.summaryID).toggle(content !== '').find('ul').html(content);
	};

	/**
	 * Performs the ajax validation request.
	 * This method is invoked internally to trigger the ajax validation.
	 * @param form jquery the jquery representation of the form
	 * @param successCallback function the function to be invoked if the ajax request succeeds
	 * @param errorCallback function the function to be invoked if the ajax request fails
	 */
	$.fn.yiiactiveform.validate = function (form, successCallback, errorCallback) {
		var $form = $(form),
			settings = $form.data('settings'),
			needAjaxValidation = false,
			messages = {};
		$.each(settings.attributes, function () {
			var value,
				msg = [];
			if (this.clientValidation !== undefined && (settings.submitting || this.status === 2 || this.status === 3)) {
				value = getAFValue($form.find('#' + this.inputID));
				this.clientValidation(value, msg, this);
				if (msg.length) {
					messages[this.id] = msg;
				}
			}
			if (this.enableAjaxValidation && !msg.length && (settings.submitting || this.status === 2 || this.status === 3)) {
				needAjaxValidation = true;
			}
		});

		if (!needAjaxValidation || settings.submitting && !$.isEmptyObject(messages)) {
			if (settings.submitting) {
				// delay callback so that the form can be submitted without problem
				setTimeout(function () {
					successCallback(messages);
				}, 200);
			} else {
				successCallback(messages);
			}
			return;
		}

		var $button = $form.data('submitObject'),
			extData = '&' + settings.ajaxVar + '=' + $form.attr('id');
		if ($button && $button.length) {
			extData += '&' + $button.attr('name') + '=' + $button.attr('value');
		}

		$.ajax({
			url : settings.validationUrl,
			type : $form.attr('method'),
			data : $form.serialize() + extData,
			dataType : 'json',
			success : function (data) {
				if (data !== null && typeof data === 'object') {
					$.each(settings.attributes, function () {
						if (!this.enableAjaxValidation) {
							delete data[this.id];
						}
					});
					successCallback($.extend({}, messages, data));
				} else {
					successCallback(messages);
				}
			},
			error : function () {
				if (errorCallback !== undefined) {
					errorCallback();
				}
			}
		});
	};

	/**
	 * Returns the configuration for the specified form.
	 * The configuration contains all needed information to perform ajax-based validation.
	 * @param form jquery the jquery representation of the form
	 * @return object the configuration for the specified form.
	 */
	$.fn.yiiactiveform.getSettings = function (form) {
		return $(form).data('settings');
	};

	$.fn.yiiactiveform.defaults = {
		ajaxVar: 'ajax',
		validationUrl: undefined,
		validationDelay: 200,
		validateOnSubmit : false,
		validateOnChange : true,
		validateOnType : false,
		hideErrorMessage : false,
		inputContainer : undefined,
		errorCssClass : 'error',
		successCssClass : 'success',
		validatingCssClass : 'validating',
		summaryID : undefined,
		timer: undefined,
		beforeValidateAttribute: undefined, // function (form, attribute) : boolean
		afterValidateAttribute: undefined,  // function (form, attribute, data, hasError)
		beforeValidate: undefined, // function (form) : boolean
		afterValidate: undefined,  // function (form, data, hasError) : boolean
		/**
		 * list of attributes to be validated. Each array element is of the following structure:
		 * {
		 *     id : 'ModelClass_attribute', // the unique attribute ID
		 *     model : 'ModelClass', // the model class name
		 *     name : 'name', // attribute name
		 *     inputID : 'input-tag-id',
		 *     errorID : 'error-tag-id',
		 *     value : undefined,
		 *     status : 0,  // 0: empty, not entered before,  1: validated, 2: pending validation, 3: validating
		 *     focus : undefined,  // jquery selector that indicates which element to receive input focus initially
		 *     validationDelay: 200,
		 *     validateOnChange : true,
		 *     validateOnType : false,
		 *     hideErrorMessage : false,
		 *     inputContainer : undefined,
		 *     errorCssClass : 'error',
		 *     successCssClass : 'success',
		 *     validatingCssClass : 'validating',
		 *     enableAjaxValidation : true,
		 *     enableClientValidation : true,
		 *     clientValidation : undefined, // function (value, messages, attribute) : client-side validation
		 *     beforeValidateAttribute: undefined, // function (form, attribute) : boolean
		 *     afterValidateAttribute: undefined,  // function (form, attribute, data, hasError)
		 * }
		 */
		attributes : []
	};
	
	/**
	 * YiiActiveRecord extension
	 *
	 * @author Benjamin Bytheway
	 */
	
	/**
	 * Checks to see if the form is valid.  Returns
	 * true or false.
	 *
	 * @param form the form jQuery object
	 * @return messages the json objects that are associated with the
	 * invalid form questions
	 */
	$.fn.yiiactiveform.getMessages = function (form) {
		var $form = $(form);
		var settings = $form.data('settings');
		
		var messages = {};
		
		$.each(settings.attributes, function () {
			var msg = [];
			
			if (this.clientValidation != undefined) {
				//var value = $('#' + this.inputID, $form).val();
				var value = getAFValue($form.find('#' + this.inputID));
				this.clientValidation(value, msg, this);
				
				if (msg.length) {
					messages[this.id] = msg;
				}
			}
		});
		
		return messages;
	};
	
	/**
	 * Checks to see if the form is valid.  Returns
	 * true or false.
	 *
	 * @param form the form jQuery object
	 * @return boolean whether the form is valid
	 */
	$.fn.yiiactiveform.isValid = function (form) {
		var $form = $(form);
		
		var messages = $.fn.yiiactiveform.getMessages($form);
		
		if (!$.isEmptyObject(messages)) {
			return false;
		}
		
		return true;
	};
	
	/**
	 *
	 * 
	 *
	 *
	 */
	$.fn.yiiactiveform.submit = function (form) {
		var $form = $(form);
		
		var settings = $form.data('settings');
		var faas_settings = $.fn.faas('getSettings');
		
		var beforeValidation = true;
		
		if (faas_settings.e.beforeValidationCallback) {
			beforeValidation = faas_settings.e.beforeValidationCallback();
		}
		
		if (beforeValidation) {
			var formValid = $.fn.yiiactiveform.isValid($form);
			var validationMessages = $.fn.yiiactiveform.getMessages($form);
			
			var afterValidation = true;
			if (faas_settings.e.afterValidationCallback) {
				afterValidation = faas_settings.e.afterValidationCallback(formValid, validationMessages);
			}
			
			if (formValid) {
				if (afterValidation) {
					$.fn.yiiactiveform.submitAJAX($form);
				}
				return false;
			} else {
				if (afterValidation) {
          errorFields = "";
					$.each(settings.attributes, function(){
						$.fn.yiiactiveform.updateInput(this, validationMessages, $form);
					});
					$.fn.yiiactiveform.updateSummary($form, validationMessages);
          if (window.s_adbadobefaas && errorFields != "") {
                window.s_adbadobefaas.prop31 = errorFields;
                window.s_adbadobefaas.pageName = window.s_adbadobefaas.pageName + ':validation errors';
                window.s_adbadobefaas.t();
          }
				}
				return false;
			}
		} else {
			return false;
		}
	};
	
	/**
	 *
	 * 
	 *
	 *
	 */
	$.fn.yiiactiveform.submitAJAX = function (form) {
                
        var $form = $(form);
		
		var messages = $.fn.yiiactiveform.getMessages($form);
		var settings = $form.data('settings');
		
		var faas_settings = $.fn.faas("getSettings");
		
		if (faas_settings.e.beforeSubmitCallback) {
			faas_settings.e.beforeSubmitCallback();
		}
		
		//$.fn.yiiactiveform.updateSummary($form, messages);
		
		$.fn.faas("overlay", $form);
		
		// submit form through ajax
		$.ajax({
			type : 'GET',
			url : 'https://apps.enterprise.adobe.com/faas/service/3.0.0/submit',
			data : $form.serialize() + '&' + settings.ajaxVar + '=' + $form.attr('id'),// + '&' + faas_settings_string,
			dataType : 'jsonp',
			canChunk : true,
			chunkStatus: function (status, arguments) {
				if (faas_settings.e.submitChunkStatusCallback) {
					faas_settings.e.submitChunkStatusCallback(status, arguments);
				}
			},
			success : function (data) {
				
				if (data != null && typeof data == 'object') {
					if (data.success) {
						$.fn.faas("overlayRemove", $form);
						
						if (data.data['faas_form_'+faas_settings.id+'_status'])
							jQuery.fn.faas("setCookie", 'faas_form_'+faas_settings.id+'_status', data.data['faas_form_'+faas_settings.id+'_status'], 30);
						if (data.data['faas_form_'+faas_settings.id+'_hash'])
							jQuery.fn.faas("setCookie", 'faas_form_'+faas_settings.id+'_hash', data.data['faas_form_'+faas_settings.id+'_hash'], 30);
						if (data.data.lv_id)
							jQuery.fn.faas("setCookie", 'lv_id', data.data.lv_id, 30);
						if (data.data.s_iid)
							jQuery.fn.faas("setCookie", 's_iid', data.data.s_iid, 30);
						if (data.data.s_cid)
							jQuery.fn.faas("setCookie", 's_cid', data.data.s_cid, 30);
						if (data.data.s_rtid)
							jQuery.fn.faas("setCookie", 's_rtid', data.data.s_rtid, 30);
						if (data.data.s_osc)
							jQuery.fn.faas("setCookie", 's_osc', data.data.s_osc, 30);
						if (data.data.s_ftid)
							jQuery.fn.faas("setCookie", 's_ftid', data.data.s_ftid, 30);
						
            //---------------------------------------------------------------
            // send information to Adobe Analytics Global Production
            //---------------------------------------------------------------
            var 
              w = window,
              s = w.s_adbadobenonacdc,
              linkTrackVarsBackup = '',
              linkTrackEventsBackup = '',
              variableOverrides,
              waitForAnalytics = false,
              s_faas = w.s_adbadobefaas;

            if (s_faas) {
              waitForAnalytics = true;
              s_faas.pageName = "faaswc:FaaS Form Submission";
              s_faas.prop42 = (data.data.destination)? data.data.destination : '';
              s_faas.t();
            }

            if (s && s.tl) {
              // backup link track stuff
              linkTrackVarsBackup = s.linkTrackVars;
              linkTrackEventsBackup = s.linkTrackEvents;

              // set link track stuff for this call
              s.linkTrackVars = (linkTrackVarsBackup === 'None' ? '' : (linkTrackVarsBackup + ',')) + 'eVar28,eVar69,eVar87,eVar88,eVar116';
              s.linkTrackEvents = (linkTrackEventsBackup === 'None' ? '' : (linkTrackEventsBackup + ',')) + 'event109';
              
              // send custom link with event and information
              variableOverrides = {
                eVar28: s.eVar28,
                eVar69: s.pageName,
                eVar87: $('.faas-form .18 select').val(),
                eVar88: $('.faas-form .20 select').val(),
                eVar116: w.formId,
                events: (s.events ? s.events + ',' : '') + 'event109'
              };

              // if (s.tl_s) {
              //   // synchronous ajax request
              //   s.tl_s(true, 'o', 'FaaS Form Submission', variableOverrides);
              // } else {
                waitForAnalytics = true;
                // asynchronous ajax/image request
                s.tl(true, 'o', 'FaaS Form Submission', variableOverrides);
              // }
                
              
              // restore link track stuff
              s.linkTrackVars = linkTrackVarsBackup;
              s.linkTrackEvents = linkTrackEventsBackup
                
              // forget for next time (do I need this?)
              linkTrackVarsBackup = '';
              linkTrackEventsBackup = '';

              // ThirdParty Script Google Conversion tag & Bing tag to trigger whenever event109 is triggered && url contains “edtamo=true”
              if (_satellite.getQueryParam("edtamo")) {
               
                Bing_img = new Image();
                Bing_img.src = 'http://bat.bing.com/action/0?ti=5626338&Ver=2&ec=EC&ea=FormSubmission&el=EC&ev=0';
                Bing_img.width = "1";
                Bing_img.height = "1";
                Bing_img.style.display = "none";
               
                Google_img = new Image();
                Google_img.src = '//www.googleadservices.com/pagead/conversion/987390658/?label=4JZrCO-EyGcQwsXp1gM&guid=ON&ascript=0';
                Google_img.width = "1";
                Google_img.height = "1";
                Google_img.style.display = "none";
              }

						}
						
						if (faas_settings.e.afterSubmitCallback) {
							faas_settings.e.afterSubmitCallback(data);
						}

						if (data.data.destination && faas_settings.ar) {
                                                    if (waitForAnalytics) {
                                                        setTimeout(function () {
                                                            w.location = data.data.destination;
                                                        }, 300);
                                                    } else {
                                                        w.location = data.data.destination;
                                                    }
						}
					} else {
						data.data = $.extend({}, messages, data.data);
					
						var hasError = false;
								
						$.each(settings.attributes, function(){
							hasError = $.fn.yiiactiveform.updateInput(this, data.data, $form) || hasError;
						});
						
						$.fn.yiiactiveform.updateSummary($form, data.data);
					}
				} else {
					data.data = messages;
					
					var hasError=false;
							
					$.each(settings.attributes, function() {
						hasError = $.fn.yiiactiveform.updateInput(this, data.data, $form) || hasError;
					});
					
					$.fn.yiiactiveform.updateSummary($form, data.data);
				}
				
				$.fn.faas("overlayRemove", $form);
			},
			error : function() {
				/*
				if (errorCallback != undefined) {
					errorCallback();
				}
				*/
				$.fn.faas("overlayRemove", $form);
			}
		});
	};

	/**
	 * jQuery faas plugin.
	 *
	 * @author Benjamin Bytheway
	 */
	var faas_settings = {
		//_fc: 1, //(f)orm-(c)ount
		//id: '', //id of the form
		l: "en_us", // (l)ocale
		d: "http://www.adobe.com", //(d)estination
		ar: true, // (a)uto-(r)edirect enabled
		test: false, // testing of the form
		q: { // (q)uestion
			//questionID: {configuration}
			// example:
			//14: {
			//	c: [
			//		2580
			//	]
			//}
		}, 
		pc: {
			1: 'js',
			2: 'faas_submission',
			3: 'sfdc',
			4: 'demandbase'
		}, // (p)re-population (c)onfiguration
		p: { // (p)re-population
			js: {
				//questionID: 'value'
				// example:
				//1: 'email@email.com'
			},
			faas_submission: {}, //none
			sfdc: {
				//contactId: $.fn.faas('getParameterByName', 'sfid')
			},
			demandbase: {} //none
		},
		as: true, // (a)uto-(s)ubmit
		o: { // (o)ptions
			//submitButtonText: "Submit"
		},
		e: { // (e)vents
//			formLoadedCallback: undefined, //function (data, ajax) {}
//			formRenderedCallback: undefined, //function (data, ajax) {} // called the when the html is on the page
//			afterYiiLoadedCallback: undefined, //function () {} // called when yiiLoads
//			beforeValidationCallback: undefined, //function () {} // called when submit button clicked, but before validation occurs
//			afterValidationCallback: undefined, //function (isValid) {} // called after validation occurs and before submission occurs
//			formErrorCallback: undefined, //function (data, ajax) {}
//			beforeSubmitCallback: undefined, //function () {} // called before submission to database
//			afterSubmitCallback: undefined, //function () {} // called after submission
//			afterEventFinishedCallback: undefined, //function(name) {} // called after a trigger is executed
//			formLoadChunkStatusCallback: undefined, //function (status, arguments) {} //called during form load when using the chunked method after each chunk is successfully sent
//			submitChunkStatusCallback: undefined //function (status, arguments) {} //called during submit when using the chunked method after each chunk is successfully sent
		},
		cookie: {},
		url: {},
		js: {}
	};
	
	var faas_methods = {
		init: function (options) {
			
			//********************************************************************
			// javascript params
			//********************************************************************
			faas_settings.js = options;
			
			//********************************************************************
			// URL params
			//********************************************************************
			var urlParams = {
				p: {
					js: {}
				}
			};
			
			var locale = $.fn.faas("getParameterByName", 'loc');
			if (locale) {
				urlParams.l = locale;
			}
			var externalCampaignId = $.fn.faas("getParameterByName", 's_cid');
			if (externalCampaignId) {
				urlParams.p.js['37'] = externalCampaignId;
			}
			var retouchCampaignId = $.fn.faas("getParameterByName", 's_rtid');
			if (retouchCampaignId) {
				urlParams.p.js['38'] = retouchCampaignId;
			}
			var onsiteCampaignId = $.fn.faas("getParameterByName", 's_osc');
			if (onsiteCampaignId) {
				urlParams.p.js['39'] = onsiteCampaignId;
			}
			var formTestCampaignId = $.fn.faas("getParameterByName", 's_ftid');
			if (formTestCampaignId) {
				urlParams.p.js['40'] = formTestCampaignId;
			}
			var productionCampaignId = $.fn.faas("getParameterByName", 's_iid');
			if (productionCampaignId) {
				urlParams.p.js['36'] = productionCampaignId;
			} else {
				urlParams.p.js['36'] = '70130000000kYe0AAE';
			}


	        var googleClickId = $.fn.faas("getParameterByName", 'gclid');
	        if (googleClickId) {
	          urlParams.p.js['109'] = googleClickId;
	        }
			
			faas_settings.url = urlParams;
			
			//********************************************************************
			// cookie params
			//********************************************************************
			var cookieParams = {
				p: {
					js: {}
				}
			};
			
			locale = $.fn.faas("getCookie", 'cms_site_lang');
			if (locale) {
				cookieParams.l = locale;
			}
			externalCampaignId = $.fn.faas("getCookie", 's_cid');
			if (externalCampaignId) {
				cookieParams.p.js['37'] = externalCampaignId;
			}
			retouchCampaignId = $.fn.faas("getCookie", 's_rtid');
			if (retouchCampaignId) {
				cookieParams.p.js['38'] = retouchCampaignId;
			}
			onsiteCampaignId = $.fn.faas("getCookie", 's_osc');
			if (onsiteCampaignId) {
				cookieParams.p.js['39'] = onsiteCampaignId;
			}
			formTestCampaignId = $.fn.faas("getCookie", 's_ftid');
			if (formTestCampaignId) {
				cookieParams.p.js['40'] = formTestCampaignId;
			}
			// We don't want to get any iid's from the cookie
			//if ($.fn.faas("getCookie", 's_iid'))
			//	cookieParams.p.js['q36'] = $.fn.faas("getCookie", 's_iid');
			
			faas_settings.cookie = cookieParams;
			
			//******************************************************************
			// merge settings
			//******************************************************************
			// set values based on importance
			$.extend(true, faas_settings, cookieParams); // least important is what the cookie says
			$.extend(true, faas_settings, urlParams); // next most important comes from the url Parameters
			
			if (options) {
				$.extend(true, faas_settings, options); // most important is what the user specifies in javascript
                                
                                //override pc settings
                                if (options.pc) {
                                    faas_settings.pc = options.pc;
                                }
			}
			
			// Set cookies for campaign tracking
			if (faas_settings.p.js['36']) {
				$.fn.faas("setCookie", 's_iid', faas_settings.p.js['36'], 30);
				$.fn.faas("setCookie", 's_iid', faas_settings.p.js['36'], 30, '/', 'adobe.com');
			}
			if (faas_settings.p.js['37']) {
				$.fn.faas("setCookie", 's_cid', faas_settings.p.js['37'], 30);
				$.fn.faas("setCookie", 's_cid', faas_settings.p.js['37'], 30, '/', 'adobe.com');
			}
			if (faas_settings.p.js['38']) {
				$.fn.faas("setCookie", 's_rtid', faas_settings.p.js['38'], 30);
				$.fn.faas("setCookie", 's_rtid', faas_settings.p.js['38'], 30, '/', 'adobe.com');
			}
			if (faas_settings.p.js['39']) {
				$.fn.faas("setCookie", 's_osc', faas_settings.p.js['39'], 30);
				$.fn.faas("setCookie", 's_osc', faas_settings.p.js['39'], 30, '/', 'adobe.com');
			}
			if (faas_settings.p.js['40']) {
				$.fn.faas("setCookie", 's_ftid', faas_settings.p.js['40'], 30);
				$.fn.faas("setCookie", 's_ftid', faas_settings.p.js['40'], 30, '/', 'adobe.com');
			}
			
			//******************************************************************
			// Special case of treatmentId
			//******************************************************************
			var treatmentId = '';
			if ($.fn.faas("getParameterByName", 'trackingid')) {
				treatmentId = $.fn.faas("getParameterByName", 'trackingid');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getParameterByName", 'prid')) {
				treatmentId = $.fn.faas("getParameterByName", 'prid');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getCookie", 'TID')) {
				treatmentId = $.fn.faas("getCookie", 'TID');
				faas_settings.cookie.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getParameterByName", 'promoid')) {
				treatmentId = $.fn.faas("getParameterByName", 'promoid');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getParameterByName", 'sdid')) {
				treatmentId = $.fn.faas("getParameterByName", 'sdid');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getParameterByName", 'pss')) {
				treatmentId = $.fn.faas("getParameterByName", 'pss');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getParameterByName", 'campaignid')) {
				treatmentId = $.fn.faas("getParameterByName", 'campaignid');
				faas_settings.url.p.js['32'] = treatmentId;
			} else if ($.fn.faas("getCookie", 'ANYPROMO')) {
				treatmentId = $.fn.faas("getCookie", 'ANYPROMO');
				faas_settings.cookie.p.js['32'] = treatmentId;
			} else {
				treatmentId = 'unknown';
			}
			
			if (treatmentId) {
				faas_settings.p.js['32'] = treatmentId;
			}
			
                        //******************************************************************
                        // Special case of AssignedId
                        //******************************************************************
                        var assigned_id = '';
                        if ($.fn.faas("getParameterByName", 'assigned_id')) {
                          assigned_id = $.fn.faas("getParameterByName", 'assigned_id');
                          faas_settings.url.p.js['119'] = assigned_id;
                        }

                        if (assigned_id) {
                          faas_settings.p.js['119'] = assigned_id;
                        }
      
			//******************************************************************
			// Salesforce Contact id
			//******************************************************************
			faas_settings.p.sfdc.contactId = $.fn.faas('getParameterByName', 'sfid');
			
			var formCount = 0;
			
			return this.each(function () {
				var $this = $(this);
				
				$.fn.faas("overlay", $this);
				
				formCount++;
				faas_settings._fc = formCount;
				
				if (faas_settings.test) {
					$.ajax({
						type: "GET",
						url: "https://apps.enterprise.adobe.com/faas/service/3.0.0/test",
						data: $.fn.faas('param', faas_settings, false, false, false),
						dataType: "jsonp",
						canChunk: true,
						chunkStatus: function (status, arguments) {
							// do nothing
						},
						success: function (data, textStatus, jqXHR) {
							$.fn.faas('overlayRemove', $this, function () {
								$this.html(data.data);	
								$this.append($('<div class="faas-form-settings" style="display:none;">').text(JSON.stringify(faas_settings)));
							});
						},
						error: function (jqXHR, textStatus, errorThrown) {
							$.fn.faas('overlayRemove', $this, function () {
								$this.append($('<div class="faas-form-settings" style="display:none;">').text(JSON.stringify(faas_settings)));
							});
						}
					});
				} else {
					$.ajax({
						type: "GET",
						url: "https://apps.enterprise.adobe.com/faas/service/3.0.0/renderJSON",
						data: $.fn.faas('param', faas_settings, false, false, false),
						dataType: "jsonp",
						canChunk: true,
						chunkStatus: function (status, arguments) {
							if (faas_settings.e.formLoadChunkStatusCallback) {
								faas_settings.e.formLoadChunkStatusCallback(status, arguments);
							}
						},
						success: function (data, textStatus, jqXHR) {
							if (data.success) {
								if (typeof data.data == 'object') {
									if (data.data['faas_form_'+faas_settings.id+'_status'])
										jQuery.fn.faas("setCookie", 'faas_form_'+faas_settings.id+'_status', data.data['faas_form_'+faas_settings.id+'_status'], 30);
									if (data.data['faas_form_'+faas_settings.id+'_hash'])
										jQuery.fn.faas("setCookie", 'faas_form_'+faas_settings.id+'_hash', data.data['faas_form_'+faas_settings.id+'_hash'], 30);
									if (data.data.lv_id)
										jQuery.fn.faas("setCookie", 'lv_id', data.data.lv_id, 30);
									if (data.data.s_iid)
										jQuery.fn.faas("setCookie", 's_iid', data.data.s_iid, 30);
									if (data.data.s_cid)
										jQuery.fn.faas("setCookie", 's_cid', data.data.s_cid, 30);
									if (data.data.s_rtid)
										jQuery.fn.faas("setCookie", 's_rtid', data.data.s_rtid, 30);
									if (data.data.s_osc)
										jQuery.fn.faas("setCookie", 's_osc', data.data.s_osc, 30);
									if (data.data.s_ftid)
										jQuery.fn.faas("setCookie", 's_ftid', data.data.s_ftid, 30);

									if (faas_settings.e.afterSubmitCallback) {
										faas_settings.e.afterSubmitCallback(data);
									}

									if (data.data.destination && faas_settings.ar) {
										window.location = data.data.destination;
									}
								} else {
									$.fn.faas('overlayRemove', $this, function () {
										$this.empty().html(data.data);
										$this.append($('<div class="faas-form-settings" style="display:none;">').text(JSON.stringify(faas_settings)));
										
										if (faas_settings.e.formRenderedCallback) {
											faas_settings.e.formRenderedCallback(data, this);
										}
									});
									
									if (faas_settings.e.formLoadedCallback) {
										faas_settings.e.formLoadedCallback(data, this);
									}
								}
							}
							else
							{
								$.fn.faas('overlayRemove', $this, function () {
									$this.empty().html(data.data);
									$this.append($('<div class="faas-form-settings" style="display:none;">').text(JSON.stringify(faas_settings)));

									if (faas_settings.e.formRenderedCallback) {
										faas_settings.e.formRenderedCallback(data, this);
									}
								});

								if (faas_settings.e.formLoadedCallback) {
									faas_settings.e.formLoadedCallback(data, this);
								}
							}
						},
						error: function (jqXHR, textStatus, errorThrown) {
							$.fn.faas('overlayRemove', $this, function () {
								$this.append($('<div class="faas-form-settings" style="display:none;">').text(JSON.stringify(faas_settings)));
							});
						}
					});
				}
			});
		},
		overlay: function ($this) {
			var overlay = $('<div/>', {
				'class': 'faas-form-overlay'
			}).css({
				'background': 'url(https://apps.enterprise.adobe.com/files/images/ajax-loader-2.gif) no-repeat center center',
				'position': 'absolute',
				'width': '100%',
				'height': '100%', 
				'top': 0, 
				'left': 0, 
				'min-height': '200px',
				'opacity': '0.5'
			});
			
			$this.css({
				'position': 'relative',
				'min-height': '200px'
			});
			
			$this.append(overlay);
		},
		overlayRemove: function ($this, callback) {
			$this.find('.faas-form-overlay').fadeOut('fast', function () {
				$this.find('.faas-form-overlay').remove();
				if (callback) {
					callback();
				}
			});
		},
		getParameterByName: function (name) {
			//alert("getParameterByName");
			var match = RegExp('[?&]' + name + '=([^&]*)').exec(window.location.search);
			return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
		},
		setCookie: function (name, value, expire, path, domain, secure) {
			var expires = false;
			if (typeof expire === 'number') {
				expires = true;
				expireDate = new Date();
				var today = new Date();
				expireDate.setDate(today.getDate() + expire);
			}
			
			document.cookie = [
				encodeURIComponent(name), '=', encodeURIComponent(value),
				(expires)	? '; expires='+expireDate.toUTCString() : '',
				(path)		? '; path='+path : '',
				(domain)	? '; domain='+domain : '',
				(secure)	? '; secure' : '; secure'
			].join('');
		},
		getCookie: function (name) {
			//alert("getCookie");
			var i,x,y,ARRcookies=document.cookie.split(";");
			for (i=0;i<ARRcookies.length;i++)
			{
				x=ARRcookies[i].substr(0,ARRcookies[i].indexOf("="));
				y=ARRcookies[i].substr(ARRcookies[i].indexOf("=")+1);
				x=x.replace(/^\s+|\s+$/g,""); //trim
				if (x==name)
				{
					return decodeURIComponent(y);
				}
			}
		},
		getSettings: function () {
			return faas_settings;
		},
		onSubmit: function () {
			//alert("onSubmit");
		},
		param: function ( a, traditional, includeObjectName, includeFunctions ) {
			var s = [],
				add = function( key, value ) {
					// If value is a function, invoke it and return its value
					//value = jQuery.isFunction( value ) ? value() : value;
					if (jQuery.isFunction( value )) {
						value = "";
						if (includeFunctions) {
							s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
						} else {
							// don't add anything
							//s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );	
						}
					} else {
						s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
					}
				};

			// Set traditional to true for jQuery <= 1.3.2 behavior.
			if ( traditional === undefined ) {
				traditional = jQuery.ajaxSettings.traditional;
			}
			
			if ( includeObjectName === undefined ) {
				includeObjectName = false;
			}

			// If an array was passed in, assume that it is an array of form elements.
			if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
				// Serialize the form elements
				jQuery.each( a, function() {
					add( this.name, this.value );
				});

			} else {
				// If traditional, encode the "old" way (the way 1.3.2 or older
				// did it), otherwise encode params recursively.
				
				var objectName = ((includeObjectName) ? $.fn.faas("getObjectName",a) : "");
				
				for ( var prefix in a ) {
					$.fn.faas('buildParams', ((includeObjectName) ? (objectName + "[" + prefix + "]") : prefix), a[ prefix ], traditional, add, includeObjectName );
				}
			}

			// Return the resulting serialization
			return s.join( "&" ).replace( /%20/g, "+" );
		},
		buildParams: function ( prefix, obj, traditional, add, includeObjectName ) {
			if ( jQuery.isArray( obj ) ) {
				// Serialize array item.
				jQuery.each( obj, function( i, v ) {
					if ( traditional || /\[\]$/.test( prefix ) ) {
						// Treat each array item as a scalar.
						add( prefix, v );

					} else {
						// If array item is non-scalar (array or object), encode its
						// numeric index to resolve deserialization ambiguity issues.
						// Note that rack (as of 1.0.0) can't currently deserialize
						// nested arrays properly, and attempting to do so may cause
						// a server error. Possible fixes are to modify rack's
						// deserialization algorithm or to provide an option or flag
						// to force array serialization to be shallow.
						$.fn.faas('buildParams', prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, traditional, add, includeObjectName );
					}
				});

			} else if ( !traditional && typeof obj === "object" ) {
				// Serialize object item.
				for ( var name in obj ) {
					$.fn.faas('buildParams', prefix + ((includeObjectName) ? ("[" + $.fn.faas("getObjectName",obj) + "]") : "") + "[" + name + "]", obj[ name ], traditional, add, includeObjectName );
				}

			} else {
				// Serialize scalar item.
				add( prefix, obj );
			}
		},
		getObjectName: function (obj) {
			var funcNameRegex = /function (.{1,})\(/;
			var results = (funcNameRegex).exec((obj).constructor.toString());
			return (results && results.length > 1) ? results[1] : "";
		}
	};
	
	$.fn.faas = function (method) {
		if (faas_methods[method]) {
			return faas_methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === 'object' || !method) {
			return faas_methods.init.apply(this, arguments);
		} else {
			$.error('Method' + method + 'does not exist on jQuery.faas');
		}
	};

})(jQuery);